/***************************************************************************
*   Copyright (C) 2004-2005 by Thomas Fischer                             *
*   fischer@unix-ag.uni-kl.de                                             *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#ifndef BIBTEXBIBTEXENTRY_H
#define BIBTEXBIBTEXENTRY_H

#include <qvaluelist.h>
#include <qstringlist.h>

#include <bibtexelement.h>
#include <bibtexentryfield.h>

class QString;
class QStringList;

namespace BibTeX
{

    class BibTeXEntry : public BibTeXElement
    {
    public:
        typedef QValueList<BibTeX::BibTeXEntryField*> EntryFields;

        enum EntryType {etArticle, etBook, etBooklet, etCollection, etElectronic, etInBook, etInCollection, etInProceedings, etManual, etMastersThesis, etMisc, etPhDThesis, etProceedings, etTechReport, etUnpublished, etUnknown};

        enum FieldRequireStatus {frsRequired, frsOptional, frsIgnored};

        BibTeXEntry( );
        BibTeXEntry( const EntryType entryType, const QString &id );
        BibTeXEntry( const QString& entryTypeString, const QString& id );
        BibTeXEntry( BibTeXEntry *other );
        virtual ~BibTeXEntry();

        void setEntryType( const EntryType elementType );
        void setEntryTypeString( const QString& elementTypeString );
        EntryType entryType() const;
        QString entryTypeString();
        void setId( const QString& id );
        QString id() const;

        bool containsPattern( const QString& pattern, bool caseSensitive = false );

        bool addField( BibTeXEntryField *field );
        BibTeX::BibTeXEntryField* getField( const BibTeXEntryField::FieldType fieldType );
        BibTeX::BibTeXEntryField* getField( const QString& fieldName );
        bool deleteField( const BibTeX::BibTeXEntryField::FieldType fieldType );
        bool deleteField( const QString &fieldName );

        EntryFields::iterator begin();
        EntryFields::iterator end();
        int getFieldCount() const;
        void clearFields();

        void copyFrom( BibTeX::BibTeXEntry *other );

        static QString entryTypeToString( const EntryType entryType );
        static EntryType entryTypeFromString( const QString &entryTypeString );
        static BibTeXEntry::FieldRequireStatus getRequireStatus( BibTeXEntry::EntryType entryType, BibTeXEntryField::FieldType fieldType );

    private:
        EntryType m_entryType;
        QString m_entryTypeString;
        QString m_id;
        EntryFields m_fields;
    };

}

#endif
