// $id$

/***************************************************************************
 *   Copyright (C) 2004-2005 by Thomas Fischer                             *
 *   fischer@unix-ag.uni-kl.de                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <bibtexxsltransform.h>

#include "bibtexstring.h"

#define max(a,b) ((a)>(b)?(a):(b))

namespace BibTeX
{

    BibTeXString::BibTeXString( const QString &key )
            : BibTeXElement(), m_key( key ), m_value( new BibTeXValue() )
    {
        // nothing
    }

    BibTeXString::BibTeXString( BibTeXString *other )
    {
        m_key = other->m_key;
        delete m_value;
        m_value = new BibTeXValue( other->m_value );
    }

    BibTeXString::~BibTeXString()
    {
        delete m_value;
    }

    void BibTeXString::setKey( const QString &key )
    {
        m_key = key;
    }

    QString BibTeXString::key()
    {
        return m_key;
    }

    BibTeXValue *BibTeXString::value()
    {
        return m_value;
    }

    void BibTeXString::setValue( BibTeXValue *value )
    {
        if ( value != m_value )
        {
            delete m_value;

            if ( value != NULL )
                m_value = new BibTeXValue( value );
            else
                m_value = NULL;
        }
    }

    bool BibTeXString::containsPattern( const QString& pattern, bool caseSensitive )
    {
        bool result = m_key.find( pattern, 0, caseSensitive ) != -1;
        result |= m_value->containsPattern( pattern, caseSensitive );

        return result;
    }

}
