/***************************************************************************
*   Copyright (C) 2004-2005 by Thomas Fischer                             *
*   fischer@unix-ag.uni-kl.de                                             *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#include "bibtexvalue.h"

namespace BibTeX
{
    BibTeXValueItem::BibTeXValueItem( const QString& text, bool isStringKey )
            : m_text( text ), m_isStringKey( isStringKey )
    {
        // nothing
    }

    BibTeXValueItem::BibTeXValueItem( BibTeXValueItem *other )
            : m_text( other->m_text ), m_isStringKey( other->m_isStringKey )
    {
        // nothing
    }

    QString BibTeXValueItem::text() const
    {
        return m_text;
    }

    bool BibTeXValueItem::isStringKey() const
    {
        return m_isStringKey;
    }

    void BibTeXValueItem::setText( const QString &text )
    {
        m_text = text;
    }

    void BibTeXValueItem::setIsStringKey( bool isStringKey )
    {
        m_isStringKey = isStringKey;
    }

    bool BibTeXValueItem::containsPattern( const QString & pattern, bool caseSensitive ) const
    {
        return m_text.find( pattern, 0, caseSensitive ) != -1;
    }

    BibTeXValue::BibTeXValue( )
    {
        m_valueItems.setAutoDelete( TRUE );
    }

    BibTeXValue::BibTeXValue( const BibTeXValue* other )
    {
        m_valueItems.setAutoDelete( TRUE );
        for ( ValueItems::const_iterator it = other->m_valueItems.begin(); it != other->m_valueItems.end(); it++ )
            m_valueItems.append( new BibTeXValueItem( *it ) );
    }

    void BibTeXValue::add ( BibTeXValueItem* valueItem )
    {
        m_valueItems.append( valueItem );
    }

    void BibTeXValue::clear()
    {
        m_valueItems.clear();
    }

    BibTeX::BibTeXValueItem *BibTeXValue::first()
    {
        if ( m_valueItems.isEmpty() )
            return NULL;
        else
            return m_valueItems.first();
    }

    bool BibTeXValue::isEmpty() const
    {
        return m_valueItems.isEmpty();
    }

    int BibTeXValue::count() const
    {
        return m_valueItems.count();
    }

    bool BibTeXValue::containsPattern( const QString & pattern, bool caseSensitive ) const
    {
        bool result = FALSE;
        for ( ValueItems::const_iterator it = m_valueItems.begin(); !result && it != m_valueItems.end(); it++ )
            result |= ( *it ) ->containsPattern( pattern, caseSensitive );
        return result;
    }

    bool BibTeXValue::checkIsStringKey( const QString &text )
    {
        bool charsOK = TRUE;
        static QString validChars = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKMNOPQRSTUVWXYZ0123456789";

        for ( unsigned int i = 0; charsOK && ( i < text.length() );i++ )
            charsOK &= validChars.contains( text.at( i ) );

        return charsOK;
    }

    BibTeXValue::ValueItems::const_iterator BibTeXValue::begin() const
    {
        return m_valueItems.begin();
    }

    BibTeXValue::ValueItems::const_iterator BibTeXValue::end() const
    {
        return m_valueItems.end();
    }

    QString BibTeXValue::plainString() const
    {
        QString result;

        for ( ValueItems::const_iterator it = m_valueItems.begin(); it != m_valueItems.end(); it++ )
        {
            if ( !result.isEmpty() )
                result.append( " " );
            result.append( ( *it ) ->text() );
        }

        result.replace( '{', "" ).replace( '}', "" );

        return result;
    }

    QString BibTeXValue::debugString() const
    {
        QString result;

        for ( ValueItems::const_iterator it = m_valueItems.begin(); it != m_valueItems.end(); it++ )
        {
            const BibTeXValueItem *item = *it;
            if ( !result.isEmpty() )
                result.append( " " );
            result.append( "[" );
            result.append( item->text() );
            if ( item->isStringKey() )
                result.append( "|KEY" );
            result.append( "]" );
        }

        result.replace( '{', "" ).replace( '}', "" );

        return result;
    }
}
