/***************************************************************************
*   Copyright (C) 2004-2005 by Thomas Fischer                             *
*   fischer@unix-ag.uni-kl.de                                             *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#ifndef BIBTEXENTRYFIELD_H
#define BIBTEXENTRYFIELD_H

#include <qobject.h>
#include <qptrlist.h>

namespace BibTeX
{
    class BibTeXValueItem
    {
    public:
        BibTeXValueItem( const QString& text, bool isStringKey );
        BibTeXValueItem( BibTeXValueItem *other );

        QString text() const;
        bool isStringKey() const;
        void setText( const QString &text );
        void setIsStringKey( bool isStringKey );

        bool containsPattern( const QString & pattern, bool caseSensitive ) const;

    private:
        QString m_text;
        bool m_isStringKey;
    };

    class BibTeXValue
    {
    public:
        typedef QPtrList<BibTeXValueItem> ValueItems;

        BibTeXValue( );
        BibTeXValue( const BibTeXValue* other );

        void add ( BibTeXValueItem* valueItem );
        void clear();
        BibTeX::BibTeXValueItem *first();

        bool isEmpty() const;
        int count() const;
        bool containsPattern( const QString & pattern, bool caseSensitive ) const;

        static bool checkIsStringKey( const QString &text );

        ValueItems::const_iterator begin() const;
        ValueItems::const_iterator end() const;

        QString plainString() const;
        QString debugString() const;

    private:
        ValueItems m_valueItems;
    };

}

#endif
