/***************************************************************************
*   Copyright (C) 2004-2005 by Thomas Fischer                             *
*   fischer@unix-ag.uni-kl.de                                             *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#include <qevent.h>
#include <qdragobject.h>
#include <qfile.h>
#include <qbuffer.h>
#include <qlistview.h>
#include <qclipboard.h>
#include <qheader.h>
#include <qtextstream.h>
#include <kio/netaccess.h>
#include <ktempfile.h>
#include <klocale.h>
#include <kurl.h>
#include <kurldrag.h>
#include <kapplication.h>
#include <kpopupmenu.h>
#include <kmessagebox.h>

#include <kbibtex_part.h>
#include <documentlistviewitem.h>
#include <bibtexfile.h>
#include <bibtexfileimporterbibtex.h>
#include <bibtexfileexporterbibtex.h>
#include <bibtexelement.h>
#include <bibtexentry.h>
#include <bibtexstring.h>
#include <bibtexcomment.h>
//#include <kbibtexentrywidget.h>
#include <entrywidget.h>
#include <commentwidget.h>
#include <stringwidget.h>
#include "documentlistview.h"

namespace KBibTeX
{
    DocumentListView::DocumentListView( KBibTeX::DocumentWidget *docWidget, bool isReadOnly, QWidget *parent, const char *name ) : KListView( parent, name ), m_docWidget( docWidget ), m_contextMenu( NULL ), m_isReadOnly( isReadOnly ), m_newElementCounter( 1 )
    {
        for ( int i = 0; i < ColumnsCount; i++ )
        {
            addColumn( Columns[ i ] );
            setColumnWidthMode( i, QListView::Manual );
        }

        setAllColumnsShowFocus( true );
        setShowSortIndicator( true );
        setSelectionMode( QListView::Extended );
        header() ->setClickEnabled ( TRUE );

        //         setDragEnabled ( true );
        //         setDragAutoScroll( true );
        setAcceptDrops( TRUE );
        //         setDropVisualizer ( TRUE );

        connect( this, SIGNAL( contextMenu( KListView *, QListViewItem *, const QPoint & ) ), this, SLOT( showBibtexListContextMenu( KListView *, QListViewItem *, const QPoint & ) ) );
        connect( this, SIGNAL( doubleClicked( QListViewItem*, const QPoint&, int ) ), this, SLOT( editElement( QListViewItem* ) ) );
        connect( this, SIGNAL( dropped( QDropEvent*, QListViewItem* ) ), this, SLOT( slotDropped( QDropEvent*, QListViewItem* ) ) );
    }

    DocumentListView::~DocumentListView()
    {
        // nothing
    }

    void DocumentListView::setListViewContextMenu( KPopupMenu *menu )
    {
        m_contextMenu = menu;
    }

    bool DocumentListView::setBibTeXFile( BibTeX::BibTeXFile *bibtexFile )
    {
        m_bibtexFile = bibtexFile;
        setItems();

        return TRUE;
    }

    BibTeX::BibTeXFile* DocumentListView::getBibTeXFile( )
    {
        return m_bibtexFile;
    }

    void DocumentListView::setItems()
    {
        QApplication::setOverrideCursor( Qt::waitCursor );
        setUpdatesEnabled( FALSE );

        clear();
        for ( unsigned int i = 0; i < m_bibtexFile->count(); i++ )
        {
            BibTeX::BibTeXElement *element = m_bibtexFile->at( i );
            QListViewItem *item = new DocumentListViewItem( element, this );
            item->setVisible( m_filter.isEmpty() || element->containsPattern( m_filter ) );
        }
        //     sort();

        setUpdatesEnabled( TRUE );
        triggerUpdate();
        QApplication::restoreOverrideCursor();
        //     triggerUpdate();
    }

    void DocumentListView::updateVisiblity()
    {
        QListViewItemIterator it( this );
        while ( it.current() )
        {
            DocumentListViewItem * kblvi = dynamic_cast<DocumentListViewItem*>( it.current() );
            BibTeX::BibTeXElement *element = kblvi->element();
            bool notFiltered = m_filter.isEmpty() || element ->containsPattern( m_filter );

            BibTeX::BibTeXString *string = dynamic_cast<BibTeX::BibTeXString*>( element );
            if ( string != NULL )
                kblvi->setVisible( notFiltered && kbibtexsettings.showStrings );
            else
            {
                BibTeX::BibTeXComment *comment = dynamic_cast<BibTeX::BibTeXComment*>( element );
                if ( comment != NULL )
                    kblvi->setVisible( notFiltered && kbibtexsettings.showComments );
                else
                    kblvi->setVisible( notFiltered );
            }

            it++;
        }
    }

    void DocumentListView::deferredInitialization()
    {
        setSorting( kbibtexsettings.mainListSortingColumn );
        for ( int c = 0; c < 5; c++ )
            setColumnWidth( c, kbibtexsettings.mainListColumnWidth[ c ] );

        connect( header(), SIGNAL( sizeChange( int, int, int ) ), this, SLOT( columnSizeChange() ) );
        connect( header(), SIGNAL( clicked( int ) ), this, SLOT( setSortingColumn( int ) ) );
    }

    void DocumentListView::deleteSelected()
    {
        QListViewItemIterator it( this, QListViewItemIterator::Selected );
        QListViewItem *above = it.current() ->itemAbove();
        while ( it.current() )
        {
            DocumentListViewItem * kblvi = dynamic_cast<DocumentListViewItem*>( it.current() );
            m_bibtexFile->deleteElement( kblvi->element() );
            it++;
            takeItem( kblvi );
            delete ( kblvi );
        }

        if ( above )
            ensureItemVisible( above );

        emit modified();
    }

    void DocumentListView::copySelected()
    {
        kapp->clipboard() ->setText( selectedAsString() );
    }

    void DocumentListView::copyRefSelected()
    {
        QString refs;
        QListViewItemIterator it( this, QListViewItemIterator::Selected );
        while ( it.current() )
        {
            DocumentListViewItem * kblvi = dynamic_cast<DocumentListViewItem*>( it.current() );
            BibTeX::BibTeXEntry *entry = dynamic_cast<BibTeX::BibTeXEntry*>( kblvi->element() );
            if ( entry != NULL )
            {
                if ( !refs.isEmpty() )
                    refs.append( "," );
                refs.append( entry->id() );
            }
            it++;
        }

        kapp->clipboard() ->setText( QString( "\\cite{%1}" ).arg( refs ) );
    }

    void DocumentListView::cutSelected()
    {
        copySelected();
        deleteSelected();
    }

    bool DocumentListView::paste()
    {
        QListViewItem * lvi = selectedItem();
        if ( lvi == NULL )
            lvi = currentItem();

        DocumentListViewItem * dlvi = NULL;
        if ( lvi != NULL )
            dlvi = dynamic_cast<KBibTeX::DocumentListViewItem *>( lvi );

        return appendText( kapp->clipboard() ->text(), dlvi != NULL ? dlvi->element() : NULL );
    }


    void DocumentListView::editElement( QListViewItem * item )
    {
        QDialog::DialogCode dialogResult = QDialog::Rejected;

        if ( item == NULL )
            item = selectedItem();
        if ( item == NULL )
            item = currentItem();

        if ( item != NULL )
        {
            DocumentListViewItem * lvi = dynamic_cast<KBibTeX::DocumentListViewItem*>( item );
            if ( lvi != NULL )
            {
                BibTeX::BibTeXEntry * entry = dynamic_cast<BibTeX::BibTeXEntry*>( lvi->element() );
                if ( entry )
                    dialogResult = KBibTeX::EntryWidget::execute( entry, m_bibtexFile, m_isReadOnly );
                else
                {
                    BibTeX::BibTeXComment * comment = dynamic_cast<BibTeX::BibTeXComment*>( lvi->element() );
                    if ( comment )
                        dialogResult = KBibTeX::CommentWidget::execute( comment, m_isReadOnly );
                    else
                    {
                        BibTeX::BibTeXString* string = dynamic_cast<BibTeX::BibTeXString*>( lvi->element() );
                        if ( string )
                            dialogResult = KBibTeX::StringWidget::execute( string, m_isReadOnly );
                    }
                }

                if ( dialogResult == QDialog::Accepted )
                    lvi->updateItem();
            }
        }

        if ( dialogResult == QDialog::Accepted )
            emit modified();
    }

    void DocumentListView::filterText( const QString & text )
    {
        m_filter = text;
        updateVisiblity();
    }

    void DocumentListView::setReadOnly( bool isReadOnly )
    {
        m_isReadOnly = isReadOnly;
    }

    void DocumentListView::showBibtexListContextMenu( KListView *, QListViewItem *, const QPoint &p )
    {
        if ( m_contextMenu != NULL )
        {
            emit selectionChanged();
            m_contextMenu->popup( p );
        }
    }

    void DocumentListView::setSortingColumn( int column )
    {
        setSortColumn( column );
    }

    void DocumentListView::columnSizeChange()
    {
        for ( int i = 0; i < 5; i++ )
            kbibtexsettings.mainListColumnWidth[ i ] = columnWidth( i );
    }

    bool DocumentListView::acceptDrag( QDropEvent* event ) const
    {
        return QTextDrag::canDecode( event ) || QUriDrag::canDecode( event );;
    }

    void DocumentListView::slotDropped( QDropEvent* event, QListViewItem* item )
    {
        QString text;
        QStrList urlList;

        if ( QUriDrag::decode( event, urlList ) )
        {
            QString url = urlList.at ( 0 );
            // FIXME
            //             m_docWidget->openURL( KURL( url ) );
        }
        else if ( QTextDrag::decode( event, text ) )
        {
            event->accept( TRUE );
            DocumentListViewItem * dlvi = dynamic_cast<KBibTeX::DocumentListViewItem *>( item );
            appendText( text, dlvi != NULL ? dlvi->element() : NULL );
        }
    }

    // void DocumentListView::contentsDragEnterEvent( QDragEnterEvent * event )
    // {
    //     KListView::contentsDragEnterEvent( event );
    //     qDebug( "contentsDragEnterEvent" );
    //     event->accept( FALSE );
    // }

    // void DocumentListView::contentsDropEvent( QDropEvent * event )
    // {
    //     KListView::contentsDropEvent( event );
    //     //     dropEvent( event );
    //     qDebug( "ContentsDropEvent" );
    //     event->accept( FALSE );
    // }

    // void DocumentListView::startDrag()
    // {
    //     QDragObject * d = new QTextDrag( selectedAsString(), this );
    //     d->dragCopy();
    //     // do NOT delete d.
    // }


    bool DocumentListView::appendText( const QString &text, BibTeX::BibTeXElement *before )
    {
        bool result = FALSE;

        QBuffer buffer;
        buffer.open( IO_WriteOnly );
        QTextStream stream( &buffer );
        stream.setEncoding( QTextStream::UnicodeUTF8 );
        stream << text;
        buffer.close();

        buffer.open( IO_ReadOnly );
        BibTeX::BibTeXFileImporter *importer = new BibTeX::BibTeXFileImporterBibTeX();
        BibTeX::BibTeXFile *toBeInserted = importer->load( &buffer );
        delete importer;
        buffer.close();

        if ( toBeInserted->count() >= 1 )
        {
            m_bibtexFile->append( toBeInserted, before );

            setItems();
            emit modified();

            result = TRUE;
        }
        else
            KMessageBox::error ( this, i18n( "The clipboard's text is not a valid BibTeX text" ), i18n( "Pasting text failed" ) );


        delete toBeInserted;
        return result;
    }

    // void DocumentListView::appendBibTeXFile( BibTeX::BibTeXFile * bibtexfile )
    // {
    //     m_bibtexfile->append( bibtexfile );
    // }

    QString DocumentListView::selectedAsString()
    {
        QBuffer buffer;
        buffer.open( IO_WriteOnly );
        BibTeX::BibTeXFileExporterBibTeX * exporter = new BibTeX::BibTeXFileExporterBibTeX();
        exporter->setEncoding( BibTeX::BibTeXFile::encLaTeX );
        QListViewItemIterator it( this, QListViewItemIterator::Selected );
        while ( it.current() )
        {
            DocumentListViewItem * kblvi = dynamic_cast<DocumentListViewItem*>( it.current() );
            exporter->save( &buffer, kblvi->element() );
            it++;
        }
        delete exporter;
        buffer.close();

        buffer.open( IO_ReadOnly );
        QTextStream in( &buffer );
        in.setEncoding( QTextStream::UnicodeUTF8 );
        QString result = in.read();
        buffer.close();

        return result;
    }

    /*    int DocumentListView::addSelectedToFile( BibTeX::BibTeXFile *bibtexfile )
        {
            int result = 0;
            QListViewItemIterator it( this, QListViewItemIterator::Selected );
            while ( it.current() )
            {
                DocumentListViewItem * kblvi = dynamic_cast<DocumentListViewItem*>( it.current() );
                BibTeX::BibTeXElement *element = kblvi->element();

                BibTeX::BibTeXEntry *entry = dynamic_cast<BibTeX::BibTeXEntry*>( element );
                if ( entry )
                {
                    bibtexfile->appendElement( new BibTeX::BibTeXEntry( entry ) );
                    result++;
                }
                else
                {
                    BibTeX::BibTeXString *string = dynamic_cast<BibTeX::BibTeXString*>( element );
                    if ( string )
                    {
                        bibtexfile->appendElement( new BibTeX::BibTeXString( string ) );
                        result++;
                    }
                    else
                    {
                        BibTeX::BibTeXComment *comment = dynamic_cast<BibTeX::BibTeXComment*>( element );
                        if ( comment )
                        {
                            bibtexfile->appendElement( new BibTeX::BibTeXComment( comment ) );
                            result++;
                        }
                    }
                }

                it++;
            }

                emit modified();
            return result;
        }*/
}

#include "documentlistview.moc"
