/***************************************************************************
*   Copyright (C) 2004-2005 by Thomas Fischer                             *
*   fischer@unix-ag.uni-kl.de                                             *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#ifndef DocumentListView_H
#define DocumentListView_H

#include <klistview.h>

#include <bibtexfileimporter.h>
#include <settings.h>
#include <bibtexfile.h>

class QString;
class KURL;
class KPopupMenu;
// class BibTeXFileImporter;

namespace KBibTeX
{
    class DocumentWidget;
    class DocumentListViewItem;

    static const QString Columns[] =
        {
            QString( i18n( "Type" ) ), QString( i18n( "Id" ) ), QString( i18n( "Title" ) ), QString( i18n( "Author/Editor/Publisher" ) ), QString( i18n( "Year" ) )
        };
    static const int ColumnsCount = sizeof( Columns ) / sizeof( Columns[ 0 ] );

    class DocumentListView : public KListView
    {
        Q_OBJECT
    public:
        DocumentListView( KBibTeX::DocumentWidget *docWidget, bool isReadOnly, QWidget *parent = 0, const char *name = 0 );
        ~DocumentListView();

        void setListViewContextMenu( KPopupMenu *menu );

        bool setBibTeXFile( BibTeX::BibTeXFile *bibtexFile );
        BibTeX::BibTeXFile* getBibTeXFile( );

        void deleteSelected();
        void copySelected();
        void copyRefSelected();
        void cutSelected();
        bool paste();
        //         int addSelectedToFile( BibTeX::BibTeXFile *bibtexfile );

        //     BibTeX::BibTeXFile *selectedElements();

        //     void enabledActions();

        void updateVisiblity();

        void deferredInitialization();

    signals:
        void modified();

    public slots:
        void editElement( QListViewItem * item = NULL );
        void filterText( const QString&text );
        //             void updateView();
        void setReadOnly( bool isReadOnly );

    private slots:
        void setSortingColumn( int column );
        void columnSizeChange();
        void showBibtexListContextMenu( KListView *, QListViewItem *, const QPoint &p );
        void slotDropped( QDropEvent* event, QListViewItem* item );

    protected:
        //     void resizeEvent( QResizeEvent * e );
        //     void contentsDragEnterEvent( QDragEnterEvent *e );
        //         void dropEvent( QDropEvent *e );
        bool acceptDrag( QDropEvent* e ) const;
        //     void contentsDropEvent( QDropEvent *e );
        //     void startDrag();

    private:
        KBibTeX::DocumentWidget *m_docWidget;
        BibTeX::BibTeXFile *m_bibtexFile;
        KPopupMenu *m_contextMenu;
        bool m_isReadOnly;

        QString m_filter;
        int m_newElementCounter;

        void setItems();
        bool appendText( const QString &text, BibTeX::BibTeXElement *before );
        //         void appendBibTeXFile( BibTeX::BibTeXFile *bibtexfile );
        QString selectedAsString();
    };
}

#endif
