/***************************************************************************
*   Copyright (C) 2004-2005 by Thomas Fischer                             *
*   fischer@unix-ag.uni-kl.de                                             *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#include <qlistview.h>
#include <klocale.h>

#include "documentlistviewitem.h"
#include "bibtexentry.h"
#include "bibtexcomment.h"
#include "bibtexstring.h"

namespace KBibTeX
{
    DocumentListViewItem::DocumentListViewItem( BibTeX::BibTeXElement *element, QListView *parent ) : KListViewItem( parent, parent->lastItem() ), m_element( element )
    {
        setTexts();
    }

    DocumentListViewItem::DocumentListViewItem( BibTeX::BibTeXElement *element, QListView *parent, QListViewItem *after ) : KListViewItem( parent, after ), m_element( element )
    {
        setTexts();
    }

    DocumentListViewItem::~DocumentListViewItem()
    {
        // nothing
    }

    BibTeX::BibTeXElement* DocumentListViewItem::element()
    {
        return m_element;
    }

    void DocumentListViewItem::updateItem()
    {
        setTexts();
    }

    void DocumentListViewItem::setTexts()
    {
        BibTeX::BibTeXEntry * entry;
        entry = dynamic_cast<BibTeX::BibTeXEntry*>( m_element );
        if ( entry )
        {
            if ( entry->entryType() != BibTeX::BibTeXEntry::etUnknown )
                setText( 0, BibTeX::BibTeXEntry::entryTypeToString( entry->entryType() ) );
            else
                setText( 0, entry->entryTypeString() );
            setText( 1, entry->id() );

            BibTeX::BibTeXEntryField *field = entry->getField( BibTeX::BibTeXEntryField::ftTitle );
            if ( field != NULL && !field->value() ->isEmpty() )
                setText( 2, field->value() ->plainString() );
            else
            {
                field = entry->getField( BibTeX::BibTeXEntryField::ftBookTitle );
                if ( field != NULL && !field->value() ->isEmpty() )
                    setText( 2, field->value() ->plainString() );
            }

            field = entry->getField( BibTeX::BibTeXEntryField::ftAuthor );
            if ( field != NULL && !field->value() ->isEmpty() )
                setText( 3, field->value() ->plainString() );
            else
            {
                field = entry->getField( BibTeX::BibTeXEntryField::ftEditor );
                if ( field != NULL && !field->value() ->isEmpty() )
                    setText( 3, field->value() ->plainString() );
                else
                {
                    field = entry->getField( BibTeX::BibTeXEntryField::ftPublisher );
                    if ( field != NULL && !field->value() ->isEmpty() )
                        setText( 3, field->value() ->plainString() );
                }
            }

            field = entry->getField( BibTeX::BibTeXEntryField::ftYear );
            if ( field != NULL && !field->value() ->isEmpty() )
                setText( 4, field->value() ->plainString() );
        }
        else
        {
            BibTeX::BibTeXComment *comment;
            comment = dynamic_cast<BibTeX::BibTeXComment*>( m_element );
            if ( comment )
            {
                setText( 0, i18n( "Comment" ) );
                QString text = comment->text();
                text.replace( '\n', ' ' );
                setText( 2, text );
            }
            else
            {
                BibTeX::BibTeXString * string;
                string = dynamic_cast<BibTeX::BibTeXString*>( m_element );
                if ( string )
                {
                    setText( 0, i18n( "String" ) );
                    setText( 1, string->key() );
                    if ( !string->value() ->isEmpty() )
                        setText( 2, string->value() ->plainString() );
                }
            }
        }
    }
}
