/***************************************************************************
*   Copyright (C) 2004-2005 by Thomas Fischer                             *
*   fischer@unix-ag.uni-kl.de                                             *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#include <qbuffer.h>

#include <bibtexfileexporterbibtex.h>
#include <bibtexfileimporterbibtex.h>
#include <settings.h>

#include "documentsourceview.h"

namespace KBibTeX
{

    DocumentSourceView::DocumentSourceView( KBibTeX::DocumentWidget *docWidget, bool isReadOnly, QWidget *parent, const char *name )
            : QTextEdit( parent, name ), m_docWidget( docWidget ), m_bibtexFile( NULL )
    {
        setReadOnly( isReadOnly );
        if ( !isReadOnly )
            connect( this, SIGNAL( textChanged() ), this, SLOT( slotModified() ) );
    }

    DocumentSourceView::~DocumentSourceView()
    {
        // nothing
    }

    bool DocumentSourceView::setBibTeXFile( BibTeX::BibTeXFile *bibtexFile )
    {
        bool result = FALSE;

        QBuffer buffer;
        buffer.open( IO_WriteOnly );
        BibTeX::BibTeXFileExporterBibTeX * exporter = new BibTeX::BibTeXFileExporterBibTeX();
        exporter->setEncoding( kbibtexsettings.encoding );
        result = exporter->save( &buffer, bibtexFile );
        delete exporter;
        buffer.close();

        if ( result )
        {
            buffer.open( IO_ReadOnly );
            QTextStream in( &buffer );
            in.setEncoding( QTextStream::UnicodeUTF8 );
            QString text = in.read();
            buffer.close();
            setText( text );

            m_bibtexFile = bibtexFile;
        }

        return result;
    }

    BibTeX::BibTeXFile* DocumentSourceView::getBibTeXFile( )
    {
        QBuffer buffer;

        buffer.open( IO_WriteOnly );
        QTextStream stream( &buffer );
        stream.setEncoding( QTextStream::UnicodeUTF8 );
        stream << text();
        buffer.close();

        buffer.open( IO_ReadOnly );
        BibTeX::BibTeXFileImporter *importer = new BibTeX::BibTeXFileImporterBibTeX();
        BibTeX::BibTeXFile *result = importer->load( &buffer );
        delete importer;
        buffer.close();

        return result;
    }

    void DocumentSourceView::slotModified()
    {
        emit modified();
    }

}
#include "documentsourceview.moc"
