/***************************************************************************
*   Copyright (C) 2004-2005 by Thomas Fischer                             *
*   fischer@unix-ag.uni-kl.de                                             *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#ifndef KBIBTEXDOCUMENTWIDGET_H
#define KBIBTEXDOCUMENTWIDGET_H

#include <qtabwidget.h>

#include <bibtexfileimporter.h>
#include <settings.h>

class QTextEdit;
class QIODevice;
class QProgressDialog;
class QListViewItem;
class KPopupMenu;

namespace KBibTeX
{
    class DocumentListView;
    class DocumentSourceView;
    class SearchBar;

    class DocumentWidget : public QTabWidget
    {
        Q_OBJECT
    public:
        DocumentWidget( bool isReadOnly, QWidget *parent = 0, const char *name = 0 );
        ~DocumentWidget();

        bool open( const QString &fileName, bool insertOnly = FALSE );
        bool open( QIODevice *iodevice, bool insertOnly = FALSE, const QString &label = QString::null, BibTeX::BibTeXFileImporter *importer = NULL );
        bool save( const QString &fileName, QStringList *errorLog = NULL );
        bool save( QIODevice *iodevice, Settings::FileFormat format = Settings::formatBibTeX, const QString &label = QString::null, QStringList *errorLog = NULL );

        bool newElement( const QString& elementType );
        void searchInternet( const QString& searchURL );
        void updateViews();

        void setAddElementMenu( KPopupMenu *menu );
        void setListViewContextMenu( KPopupMenu *menu );

        void deferredInitialization();

    signals:
        void modified();
        void listViewSelectionChanged( int numSelected );

    public slots:
        void editElement();
        void deleteElements();
        void cutElements();
        void copyElements();
        void copyRefElements();
        void pasteElements();
        void setReadOnly( bool isReadOnly );

    private:
        bool m_isReadOnly;
        QWidget *m_verticalContainer;
        KBibTeX::SearchBar *m_searchBar;
        DocumentListView *m_listViewElements;
        QLabel *m_labelPreview;
        DocumentSourceView *m_sourceView;
        BibTeX::BibTeXFile *m_bibtexfile;
        QProgressDialog *m_progressDialog;
        int m_newElementCounter;

        void setupGUI();

        void startProgress( const QString &label, QObject *progressFrom );
        void endProgress( QObject *progressFrom );

        void searchInternet( BibTeX::BibTeXElement *element, const QString& searchURL );

        void updateFromGUI();

    private slots:
        void slotShowProgress( int current, int total );
        void slotSelectionChanged();
        void slotPreviewElement( QListViewItem * item = NULL );
        void slotTabChanged( QWidget *tab );
        void slotModified();
    };

}

#endif
