/***************************************************************************
*   Copyright (C) 2004-2005 by Thomas Fischer                             *
*   fischer@unix-ag.uni-kl.de                                             *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#include <qlayout.h>
#include <qpushbutton.h>
#include <qtooltip.h>
#include <qlabel.h>
#include <qtooltip.h>

#include <kdialog.h>
#include <klocale.h>
#include <kurl.h>
#include <kurllabel.h>
#include <kiconloader.h>
#include <kapplication.h>

#include <fieldlineedit.h>
#include "entrywidgetexternal.h"

namespace KBibTeX
{

    EntryWidgetExternal::EntryWidgetExternal( BibTeX::BibTeXEntry *entry, bool isReadOnly, QWidget *parent, const char *name )
            : EntryWidgetTab( isReadOnly, parent, name ), m_entry( entry ), m_isReadOnly( isReadOnly )
    {
        setupGUI();
    }

    EntryWidgetExternal::~EntryWidgetExternal()
    {
        // nothing
    }

    void EntryWidgetExternal::installEventFilters( QWidget *listener )
    {
        m_fieldLineEditURL->installEventFilters( listener );
        m_fieldLineEditDoi->installEventFilters( listener );
    }

    bool EntryWidgetExternal::isModified()
    {
        return m_fieldLineEditURL->isModified() || m_fieldLineEditDoi->isModified();
    }

    void EntryWidgetExternal::updateGUI( BibTeX::BibTeXEntry::EntryType entryType, bool enableAll )
    {
        bool enableWidget = enableAll || BibTeX::BibTeXEntry::getRequireStatus( entryType, BibTeX::BibTeXEntryField::ftURL ) != BibTeX::BibTeXEntry::frsIgnored;
        m_fieldLineEditURL->setEnabled( enableWidget );

        enableWidget = enableAll || BibTeX::BibTeXEntry::getRequireStatus( entryType, BibTeX::BibTeXEntryField::ftDoi ) != BibTeX::BibTeXEntry::frsIgnored;
        m_fieldLineEditDoi->setEnabled( enableWidget );

        updateGUI();
    }

    void EntryWidgetExternal::apply( )
    {
        BibTeX::BibTeXValue * value = m_fieldLineEditURL->value();
        setValue( m_entry, BibTeX::BibTeXEntryField::ftURL, value );
        delete value;

        value = m_fieldLineEditDoi->value();
        setValue( m_entry, BibTeX::BibTeXEntryField::ftDoi, value );
        delete value;
    }

    void EntryWidgetExternal::reset( )
    {
        disconnect( m_fieldLineEditURL, SIGNAL( textChanged() ), this, SLOT( updateGUI() ) );
        disconnect( m_fieldLineEditDoi, SIGNAL( textChanged() ), this, SLOT( updateGUI() ) );

        BibTeX::BibTeXEntryField * field = m_entry->getField( BibTeX::BibTeXEntryField::ftURL );
        m_fieldLineEditURL->setValue( field != NULL ? field->value() : NULL );

        field = m_entry->getField( BibTeX::BibTeXEntryField::ftDoi );
        m_fieldLineEditDoi->setValue( field != NULL ? field->value() : NULL );

        updateGUI();

        connect( m_fieldLineEditURL, SIGNAL( textChanged() ), this, SLOT( updateGUI() ) );
        connect( m_fieldLineEditDoi, SIGNAL( textChanged() ), this, SLOT( updateGUI() ) );
    }

    void EntryWidgetExternal::updateWarnings( BibTeX::BibTeXEntry::EntryType entryType, QListView *listViewWarnings )
    {
        addMissingWarning( entryType, BibTeX::BibTeXEntryField::ftURL, m_fieldLineEditURL->caption(), !m_fieldLineEditURL->isEmpty(), m_fieldLineEditURL, listViewWarnings );

        addMissingWarning( entryType, BibTeX::BibTeXEntryField::ftDoi, m_fieldLineEditDoi->caption(), !m_fieldLineEditDoi->isEmpty(), m_fieldLineEditDoi, listViewWarnings );

        addFieldLineEditWarning( m_fieldLineEditURL, m_fieldLineEditURL->caption(), listViewWarnings );

        addFieldLineEditWarning( m_fieldLineEditDoi, m_fieldLineEditDoi->caption(), listViewWarnings );

        updateGUI();
    }

    void EntryWidgetExternal::openURL()
    {
        BibTeX::BibTeXValue * value = m_fieldLineEditURL->value();
        if ( value->count() == 1 && !value->first() ->isStringKey() )
        {
            KURL url( value->plainString() );
            if ( url.isValid() )
                kapp->invokeBrowser( url.prettyURL() );
            else
                m_pushButtonOpenURL->setEnabled( FALSE );
        }
    }

    void EntryWidgetExternal::openDoi()
    {
        KURL url = doiURL();
        if ( url.isValid() )
            kapp->invokeBrowser( url.prettyURL() );
        else
            m_pushButtonOpenDoi->setEnabled( FALSE );
    }

    void EntryWidgetExternal::updateGUI()
    {
        BibTeX::BibTeXValue * value = m_fieldLineEditURL->value();
        if ( value != NULL && value->count() == 1 && !value->first() ->isStringKey() )
        {
            KURL url( value->plainString() );
            m_pushButtonOpenURL->setEnabled( url.isValid() );
        }
        else
            m_pushButtonOpenURL->setEnabled( FALSE );

        KURL url = doiURL();
        m_pushButtonOpenDoi->setEnabled( url.isValid() );
    }

    void EntryWidgetExternal::gotoURL( const QString& url )
    {
        kapp->invokeBrowser( url );
    }

    void EntryWidgetExternal::setupGUI()
    {
        QGridLayout * gridLayout = new QGridLayout( this, 2, 3, KDialog::marginHint(), KDialog::spacingHint(), "gridLayout" );
        gridLayout->setRowStretch( 2, 1 );

        QLabel *label = new QLabel( QString( "%1:" ).arg( i18n( "URL" ) ), this );
        gridLayout->addWidget( label, 0, 0 );
        m_fieldLineEditURL = new KBibTeX::FieldLineEdit( i18n( "URL" ), KBibTeX::FieldLineEdit::itSingleLine, m_isReadOnly, this, "m_fieldLineEditURL" );
        gridLayout->addWidget( m_fieldLineEditURL, 0, 1 );
        label->setBuddy( m_fieldLineEditURL );
        connect( m_fieldLineEditURL, SIGNAL( textChanged() ), this, SLOT( updateGUI() ) );

        m_pushButtonOpenURL = new QPushButton( this );
        gridLayout->addWidget( m_pushButtonOpenURL, 0, 2 );
        m_pushButtonOpenURL->setIconSet( QIconSet( SmallIcon( "fileopen" ) ) );
        m_pushButtonOpenURL->setSizePolicy( QSizePolicy::Minimum, QSizePolicy::Minimum );
        QToolTip::add( m_pushButtonOpenURL, QString( i18n( "Open %1" ) ).arg( i18n( "URL" ) ) );
        connect( m_pushButtonOpenURL, SIGNAL( clicked() ), this, SLOT( openURL() ) );

        KURLLabel *urllabel = new KURLLabel( "http://www.doi.org/", QString( "%1:" ).arg( i18n( "DOI" ) ), this );
        QToolTip::add( urllabel, i18n( "Digital Object Identifier" ) );
        urllabel->setFocusPolicy( QWidget::NoFocus );
        gridLayout->addWidget( urllabel, 1, 0 );
        m_fieldLineEditDoi = new KBibTeX::FieldLineEdit( i18n( "DOI" ), KBibTeX::FieldLineEdit::itSingleLine, m_isReadOnly, this, "m_fieldLineEditDoi" );
        gridLayout->addWidget( m_fieldLineEditDoi, 1, 1 );
        urllabel->setBuddy( m_fieldLineEditDoi );
        connect( m_fieldLineEditDoi, SIGNAL( textChanged() ), this, SLOT( updateGUI() ) );
        connect( urllabel, SIGNAL( leftClickedURL( const QString& ) ), this,
                 SLOT( gotoURL( const QString& ) ) );

        m_pushButtonOpenDoi = new QPushButton( this );
        gridLayout->addWidget( m_pushButtonOpenDoi, 1, 2 );
        m_pushButtonOpenDoi->setIconSet( QIconSet( SmallIcon( "fileopen" ) ) );
        m_pushButtonOpenDoi->setSizePolicy( QSizePolicy::Minimum, QSizePolicy::Minimum );
        QToolTip::add( m_pushButtonOpenDoi, QString( i18n( "Open %1" ) ).arg( i18n( "Doi" ) ) );
        connect( m_pushButtonOpenDoi, SIGNAL( clicked() ), this, SLOT( openDoi() ) );
    }

    KURL EntryWidgetExternal::doiURL()
    {
        BibTeX::BibTeXValue * value = m_fieldLineEditDoi->value();
        if ( value != NULL && value->count() == 1 && !value->first() ->isStringKey() )
        {
            KURL result( value->plainString() );
            if ( result.isValid() )
                return result;
            else
                return KURL( QString( "http://dx.doi.org/%1" ).arg( value->plainString() ) );
        }
        else
            return KURL( "invalid" );
    }
}
#include "entrywidgetexternal.moc"
