/***************************************************************************
*   Copyright (C) 2004-2005 by Thomas Fischer                             *
*   fischer@unix-ag.uni-kl.de                                             *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#include <qlayout.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qpopupmenu.h>
#include <kiconloader.h>

#include <kdialog.h>
#include <klocale.h>

#include <fieldlineedit.h>
#include "entrywidgetpublication.h"

namespace KBibTeX
{

    EntryWidgetPublication::EntryWidgetPublication( BibTeX::BibTeXEntry *entry, bool isReadOnly, QWidget *parent, const char *name )
            : EntryWidgetTab( isReadOnly, parent, name ), m_entry( entry ), m_isReadOnly( isReadOnly )
    {
        setupGUI();
    }


    EntryWidgetPublication::~EntryWidgetPublication()
    {
        // nothing
    }

    void EntryWidgetPublication::installEventFilters( QWidget *listener )
    {
        m_fieldLineEditOrganization->installEventFilters( listener );
        m_fieldLineEditPublisher->installEventFilters( listener );
        m_fieldLineEditSchool->installEventFilters( listener );
        m_fieldLineEditInstitution->installEventFilters( listener );
        m_fieldLineEditLocation->installEventFilters( listener );
        m_fieldLineEditAddress->installEventFilters( listener );
        m_fieldLineEditJournal->installEventFilters( listener );
        m_fieldLineEditPages->installEventFilters( listener );
        m_fieldLineEditEdition->installEventFilters( listener );
        m_fieldLineEditChapter->installEventFilters( listener );
        m_fieldLineEditVolume->installEventFilters( listener );
        m_fieldLineEditNumber->installEventFilters( listener );
        m_fieldLineEditMonth->installEventFilters( listener );
        m_fieldLineEditYear->installEventFilters( listener );
        m_fieldLineEditISBN->installEventFilters( listener );
        m_fieldLineEditISSN->installEventFilters( listener );
        m_fieldLineEditCrossRef->installEventFilters( listener );
        m_fieldLineEditHowPublished->installEventFilters( listener );
    }

    bool EntryWidgetPublication::isModified()
    {
        return m_fieldLineEditOrganization->isModified() || m_fieldLineEditPublisher->isModified() || m_fieldLineEditSchool->isModified() || m_fieldLineEditInstitution->isModified() || m_fieldLineEditLocation->isModified() || m_fieldLineEditAddress->isModified() || m_fieldLineEditJournal->isModified() || m_fieldLineEditPages->isModified() || m_fieldLineEditEdition->isModified() || m_fieldLineEditChapter->isModified() || m_fieldLineEditVolume->isModified() || m_fieldLineEditNumber->isModified() || m_fieldLineEditMonth->isModified() || m_fieldLineEditYear->isModified() || m_fieldLineEditISBN->isModified() || m_fieldLineEditISSN->isModified() || m_fieldLineEditCrossRef->isModified() || m_fieldLineEditHowPublished->isModified();
    }

    void EntryWidgetPublication::updateGUI( BibTeX::BibTeXEntry::EntryType entryType, bool enableAll )
    {
        bool enableWidget = enableAll || BibTeX::BibTeXEntry::getRequireStatus( entryType, BibTeX::BibTeXEntryField::ftOrganization ) != BibTeX::BibTeXEntry::frsIgnored;
        m_fieldLineEditOrganization->setEnabled( enableWidget );

        enableWidget = enableAll || BibTeX::BibTeXEntry::getRequireStatus( entryType, BibTeX::BibTeXEntryField::ftPublisher ) != BibTeX::BibTeXEntry::frsIgnored;
        m_fieldLineEditPublisher->setEnabled( enableWidget );

        enableWidget = enableAll || BibTeX::BibTeXEntry::getRequireStatus( entryType, BibTeX::BibTeXEntryField::ftSchool ) != BibTeX::BibTeXEntry::frsIgnored;
        m_fieldLineEditSchool->setEnabled( enableWidget );

        enableWidget = enableAll || BibTeX::BibTeXEntry::getRequireStatus( entryType, BibTeX::BibTeXEntryField::ftInstitution ) != BibTeX::BibTeXEntry::frsIgnored;
        m_fieldLineEditInstitution->setEnabled( enableWidget );

        enableWidget = enableAll || BibTeX::BibTeXEntry::getRequireStatus( entryType, BibTeX::BibTeXEntryField::ftLocation ) != BibTeX::BibTeXEntry::frsIgnored;
        m_fieldLineEditLocation->setEnabled( enableWidget );

        enableWidget = enableAll || BibTeX::BibTeXEntry::getRequireStatus( entryType, BibTeX::BibTeXEntryField::ftAddress ) != BibTeX::BibTeXEntry::frsIgnored;
        m_fieldLineEditAddress->setEnabled( enableWidget );

        enableWidget = enableAll || BibTeX::BibTeXEntry::getRequireStatus( entryType, BibTeX::BibTeXEntryField::ftJournal ) != BibTeX::BibTeXEntry::frsIgnored;
        m_fieldLineEditJournal->setEnabled( enableWidget );

        enableWidget = enableAll || BibTeX::BibTeXEntry::getRequireStatus( entryType, BibTeX::BibTeXEntryField::ftPages ) != BibTeX::BibTeXEntry::frsIgnored;
        m_fieldLineEditPages->setEnabled( enableWidget );

        enableWidget = enableAll || BibTeX::BibTeXEntry::getRequireStatus( entryType, BibTeX::BibTeXEntryField::ftEdition ) != BibTeX::BibTeXEntry::frsIgnored;
        m_fieldLineEditEdition->setEnabled( enableWidget );

        enableWidget = enableAll || BibTeX::BibTeXEntry::getRequireStatus( entryType, BibTeX::BibTeXEntryField::ftChapter ) != BibTeX::BibTeXEntry::frsIgnored;
        m_fieldLineEditChapter->setEnabled( enableWidget );

        enableWidget = enableAll || BibTeX::BibTeXEntry::getRequireStatus( entryType, BibTeX::BibTeXEntryField::ftVolume ) != BibTeX::BibTeXEntry::frsIgnored;
        m_fieldLineEditVolume->setEnabled( enableWidget );

        enableWidget = enableAll || BibTeX::BibTeXEntry::getRequireStatus( entryType, BibTeX::BibTeXEntryField::ftNumber ) != BibTeX::BibTeXEntry::frsIgnored;
        m_fieldLineEditNumber->setEnabled( enableWidget );

        enableWidget = enableAll || BibTeX::BibTeXEntry::getRequireStatus( entryType, BibTeX::BibTeXEntryField::ftMonth ) != BibTeX::BibTeXEntry::frsIgnored;
        m_fieldLineEditMonth->setEnabled( enableWidget );
        m_pushButtonMonths->setEnabled( enableWidget );

        enableWidget = enableAll || BibTeX::BibTeXEntry::getRequireStatus( entryType, BibTeX::BibTeXEntryField::ftYear ) != BibTeX::BibTeXEntry::frsIgnored;
        m_fieldLineEditYear->setEnabled( enableWidget );

        enableWidget = enableAll || BibTeX::BibTeXEntry::getRequireStatus( entryType, BibTeX::BibTeXEntryField::ftISBN ) != BibTeX::BibTeXEntry::frsIgnored;
        m_fieldLineEditISBN->setEnabled( enableWidget );

        enableWidget = enableAll || BibTeX::BibTeXEntry::getRequireStatus( entryType, BibTeX::BibTeXEntryField::ftISSN ) != BibTeX::BibTeXEntry::frsIgnored;
        m_fieldLineEditISSN->setEnabled( enableWidget );

        enableWidget = enableAll || BibTeX::BibTeXEntry::getRequireStatus( entryType, BibTeX::BibTeXEntryField::ftCrossRef ) != BibTeX::BibTeXEntry::frsIgnored;
        m_fieldLineEditCrossRef->setEnabled( enableWidget );

        enableWidget = enableAll || BibTeX::BibTeXEntry::getRequireStatus( entryType, BibTeX::BibTeXEntryField::ftHowPublished ) != BibTeX::BibTeXEntry::frsIgnored;
        m_fieldLineEditHowPublished->setEnabled( enableWidget );
    }

    void EntryWidgetPublication::apply( )
    {
        BibTeX::BibTeXValue * value = m_fieldLineEditHowPublished->value();
        setValue( m_entry, BibTeX::BibTeXEntryField::ftHowPublished, value );
        delete value;

        value = m_fieldLineEditPublisher->value();
        setValue( m_entry, BibTeX::BibTeXEntryField::ftPublisher, value );
        delete value;

        value = m_fieldLineEditOrganization->value();
        setValue( m_entry, BibTeX::BibTeXEntryField::ftOrganization, value );
        delete value;

        value = m_fieldLineEditInstitution->value();
        setValue( m_entry, BibTeX::BibTeXEntryField::ftInstitution, value );
        delete value;

        value = m_fieldLineEditSchool->value();
        setValue( m_entry, BibTeX::BibTeXEntryField::ftSchool, value );
        delete value;

        value = m_fieldLineEditLocation->value();
        setValue( m_entry, BibTeX::BibTeXEntryField::ftLocation, value );
        delete value;

        value = m_fieldLineEditAddress->value();
        setValue( m_entry, BibTeX::BibTeXEntryField::ftAddress, value );
        delete value;

        value = m_fieldLineEditJournal->value();
        setValue( m_entry, BibTeX::BibTeXEntryField::ftJournal, value );
        delete value;

        value = m_fieldLineEditPages->value();
        setValue( m_entry, BibTeX::BibTeXEntryField::ftPages, value );
        delete value;

        value = m_fieldLineEditEdition->value();
        setValue( m_entry, BibTeX::BibTeXEntryField::ftEdition, value );
        delete value;

        value = m_fieldLineEditChapter->value();
        setValue( m_entry, BibTeX::BibTeXEntryField::ftChapter, value );
        delete value;

        value = m_fieldLineEditVolume->value();
        setValue( m_entry, BibTeX::BibTeXEntryField::ftVolume, value );
        delete value;

        value = m_fieldLineEditNumber->value();
        setValue( m_entry, BibTeX::BibTeXEntryField::ftNumber, value );
        delete value;

        value = m_fieldLineEditMonth->value();
        setValue( m_entry, BibTeX::BibTeXEntryField::ftMonth, value );
        delete value;

        value = m_fieldLineEditYear->value();
        setValue( m_entry, BibTeX::BibTeXEntryField::ftYear, value );
        delete value;

        value = m_fieldLineEditISBN->value();
        setValue( m_entry, BibTeX::BibTeXEntryField::ftISBN, value );
        delete value;

        value = m_fieldLineEditISSN->value();
        setValue( m_entry, BibTeX::BibTeXEntryField::ftISSN, value );
        delete value;

        value = m_fieldLineEditCrossRef->value();
        setValue( m_entry, BibTeX::BibTeXEntryField::ftCrossRef, value );
        delete value;
    }

    void EntryWidgetPublication::reset( )
    {
        BibTeX::BibTeXEntryField * field = m_entry->getField( BibTeX::BibTeXEntryField::ftHowPublished );
        m_fieldLineEditHowPublished->setValue( field != NULL ? field->value() : NULL );

        field = m_entry->getField( BibTeX::BibTeXEntryField::ftPublisher );
        m_fieldLineEditPublisher->setValue( field != NULL ? field->value() : NULL );

        field = m_entry->getField( BibTeX::BibTeXEntryField::ftOrganization );
        m_fieldLineEditOrganization->setValue( field != NULL ? field->value() : NULL );

        field = m_entry->getField( BibTeX::BibTeXEntryField::ftInstitution );
        m_fieldLineEditInstitution->setValue( field != NULL ? field->value() : NULL );

        field = m_entry->getField( BibTeX::BibTeXEntryField::ftSchool );
        m_fieldLineEditSchool->setValue( field != NULL ? field->value() : NULL );

        field = m_entry->getField( BibTeX::BibTeXEntryField::ftLocation );
        m_fieldLineEditLocation->setValue( field != NULL ? field->value() : NULL );

        field = m_entry->getField( BibTeX::BibTeXEntryField::ftAddress );
        m_fieldLineEditAddress->setValue( field != NULL ? field->value() : NULL );

        field = m_entry->getField( BibTeX::BibTeXEntryField::ftJournal );
        m_fieldLineEditJournal->setValue( field != NULL ? field->value() : NULL );

        field = m_entry->getField( BibTeX::BibTeXEntryField::ftPages );
        m_fieldLineEditPages->setValue( field != NULL ? field->value() : NULL );

        field = m_entry->getField( BibTeX::BibTeXEntryField::ftEdition );
        m_fieldLineEditEdition->setValue( field != NULL ? field->value() : NULL );

        field = m_entry->getField( BibTeX::BibTeXEntryField::ftChapter );
        m_fieldLineEditChapter->setValue( field != NULL ? field->value() : NULL );

        field = m_entry->getField( BibTeX::BibTeXEntryField::ftVolume );
        m_fieldLineEditVolume->setValue( field != NULL ? field->value() : NULL );

        field = m_entry->getField( BibTeX::BibTeXEntryField::ftNumber );
        m_fieldLineEditNumber->setValue( field != NULL ? field->value() : NULL );

        field = m_entry->getField( BibTeX::BibTeXEntryField::ftMonth );
        m_fieldLineEditMonth->setValue( field != NULL ? field->value() : NULL );

        field = m_entry->getField( BibTeX::BibTeXEntryField::ftYear );
        m_fieldLineEditYear->setValue( field != NULL ? field->value() : NULL );

        field = m_entry->getField( BibTeX::BibTeXEntryField::ftISBN );
        m_fieldLineEditISBN->setValue( field != NULL ? field->value() : NULL );

        field = m_entry->getField( BibTeX::BibTeXEntryField::ftISSN );
        m_fieldLineEditISSN->setValue( field != NULL ? field->value() : NULL );

        field = m_entry->getField( BibTeX::BibTeXEntryField::ftCrossRef );
        m_fieldLineEditCrossRef->setValue( field != NULL ? field->value() : NULL );
    }

    void EntryWidgetPublication::updateWarnings( BibTeX::BibTeXEntry::EntryType entryType, QListView *listViewWarnings )
    {
        addMissingWarning( entryType, BibTeX::BibTeXEntryField::ftHowPublished, m_fieldLineEditHowPublished->caption(), !m_fieldLineEditHowPublished->isEmpty(), m_fieldLineEditHowPublished, listViewWarnings );

        addMissingWarning( entryType, BibTeX::BibTeXEntryField::ftPublisher, m_fieldLineEditPublisher->caption(), !m_fieldLineEditPublisher->isEmpty(), m_fieldLineEditPublisher, listViewWarnings );

        addMissingWarning( entryType, BibTeX::BibTeXEntryField::ftOrganization, m_fieldLineEditOrganization->caption(), !m_fieldLineEditOrganization->isEmpty(), m_fieldLineEditOrganization, listViewWarnings );

        addMissingWarning( entryType, BibTeX::BibTeXEntryField::ftInstitution, m_fieldLineEditInstitution->caption(), !m_fieldLineEditInstitution->isEmpty(), m_fieldLineEditInstitution, listViewWarnings );

        addMissingWarning( entryType, BibTeX::BibTeXEntryField::ftSchool, m_fieldLineEditSchool->caption(), !m_fieldLineEditSchool->isEmpty(), m_fieldLineEditSchool, listViewWarnings );

        addMissingWarning( entryType, BibTeX::BibTeXEntryField::ftLocation, m_fieldLineEditLocation->caption(), !m_fieldLineEditLocation->isEmpty(), m_fieldLineEditLocation, listViewWarnings );

        addMissingWarning( entryType, BibTeX::BibTeXEntryField::ftAddress, m_fieldLineEditAddress->caption(), !m_fieldLineEditAddress->isEmpty(), m_fieldLineEditAddress, listViewWarnings );

        addMissingWarning( entryType, BibTeX::BibTeXEntryField::ftJournal, m_fieldLineEditJournal->caption(), !m_fieldLineEditJournal->isEmpty(), m_fieldLineEditJournal, listViewWarnings );

        addMissingWarning( entryType, BibTeX::BibTeXEntryField::ftPages, m_fieldLineEditPages->caption(), !m_fieldLineEditPages->isEmpty(), m_fieldLineEditPages, listViewWarnings );

        addMissingWarning( entryType, BibTeX::BibTeXEntryField::ftEdition, m_fieldLineEditEdition->caption(), !m_fieldLineEditEdition->isEmpty(), m_fieldLineEditEdition, listViewWarnings );

        addMissingWarning( entryType, BibTeX::BibTeXEntryField::ftChapter, m_fieldLineEditChapter->caption(), !m_fieldLineEditChapter->isEmpty(), m_fieldLineEditChapter, listViewWarnings );

        addMissingWarning( entryType, BibTeX::BibTeXEntryField::ftVolume, m_fieldLineEditVolume->caption(), !m_fieldLineEditVolume->isEmpty(), m_fieldLineEditVolume, listViewWarnings );

        addMissingWarning( entryType, BibTeX::BibTeXEntryField::ftNumber, m_fieldLineEditNumber->caption(), !m_fieldLineEditNumber->isEmpty(), m_fieldLineEditNumber, listViewWarnings );

        addMissingWarning( entryType, BibTeX::BibTeXEntryField::ftMonth, m_fieldLineEditMonth->caption(), !m_fieldLineEditMonth->isEmpty(), m_fieldLineEditMonth, listViewWarnings );

        addMissingWarning( entryType, BibTeX::BibTeXEntryField::ftYear, m_fieldLineEditYear->caption(), !m_fieldLineEditYear->isEmpty(), m_fieldLineEditYear, listViewWarnings );

        addMissingWarning( entryType, BibTeX::BibTeXEntryField::ftISBN, m_fieldLineEditISBN->caption(), !m_fieldLineEditISBN->isEmpty(), m_fieldLineEditISBN, listViewWarnings );

        addMissingWarning( entryType, BibTeX::BibTeXEntryField::ftISSN, m_fieldLineEditISSN->caption(), !m_fieldLineEditISSN->isEmpty(), m_fieldLineEditISSN, listViewWarnings );

        addMissingWarning( entryType, BibTeX::BibTeXEntryField::ftCrossRef, m_fieldLineEditCrossRef->caption(), !m_fieldLineEditCrossRef->isEmpty(), m_fieldLineEditCrossRef, listViewWarnings );

        addFieldLineEditWarning( m_fieldLineEditHowPublished, m_fieldLineEditHowPublished->caption(), listViewWarnings );

        addFieldLineEditWarning( m_fieldLineEditPublisher, m_fieldLineEditPublisher->caption(), listViewWarnings );

        addFieldLineEditWarning( m_fieldLineEditOrganization, m_fieldLineEditOrganization->caption(), listViewWarnings );

        addFieldLineEditWarning( m_fieldLineEditInstitution, m_fieldLineEditInstitution->caption(), listViewWarnings );

        addFieldLineEditWarning( m_fieldLineEditSchool, m_fieldLineEditSchool->caption(), listViewWarnings );

        addFieldLineEditWarning( m_fieldLineEditLocation, m_fieldLineEditLocation->caption(), listViewWarnings );

        addFieldLineEditWarning( m_fieldLineEditAddress, m_fieldLineEditAddress->caption(), listViewWarnings );

        addFieldLineEditWarning( m_fieldLineEditJournal, m_fieldLineEditJournal->caption(), listViewWarnings );

        addFieldLineEditWarning( m_fieldLineEditPages, m_fieldLineEditPages->caption(), listViewWarnings );

        addFieldLineEditWarning( m_fieldLineEditEdition, m_fieldLineEditEdition->caption(), listViewWarnings );

        addFieldLineEditWarning( m_fieldLineEditChapter, m_fieldLineEditChapter->caption(), listViewWarnings );

        addFieldLineEditWarning( m_fieldLineEditVolume, m_fieldLineEditVolume->caption(), listViewWarnings );

        addFieldLineEditWarning( m_fieldLineEditNumber, m_fieldLineEditNumber->caption(), listViewWarnings );

        addFieldLineEditWarning( m_fieldLineEditMonth, m_fieldLineEditMonth->caption(), listViewWarnings );

        addFieldLineEditWarning( m_fieldLineEditYear, m_fieldLineEditYear->caption(), listViewWarnings );

        addFieldLineEditWarning( m_fieldLineEditISBN, m_fieldLineEditISBN->caption(), listViewWarnings );

        addFieldLineEditWarning( m_fieldLineEditISSN, m_fieldLineEditISSN->caption(), listViewWarnings );

        addFieldLineEditWarning( m_fieldLineEditCrossRef, m_fieldLineEditCrossRef->caption(), listViewWarnings );
    }

    void EntryWidgetPublication::slotSetMonth( int month )
    {
        BibTeX::BibTeXValue * value = new BibTeX::BibTeXValue();
        BibTeX::BibTeXValueItem *item = new BibTeX::BibTeXValueItem( BibTeX::MonthsTriple[ month ], TRUE );
        value->add( item );
        m_fieldLineEditMonth->setValue( value );
        delete value;
    }

    void EntryWidgetPublication::setupGUI()
    {
        QGridLayout * gridLayout = new QGridLayout( this, 10, 5, KDialog::marginHint(), KDialog::spacingHint(), "gridLayout" );
        gridLayout->setColSpacing( 2, KDialog::spacingHint() * 4 );
        gridLayout->setRowStretch( 9, 1 );

        QLabel *label = new QLabel( QString( "%1:" ).arg( i18n( "Organization" ) ), this );
        gridLayout->addWidget( label, 0, 0 );
        m_fieldLineEditOrganization = new KBibTeX::FieldLineEdit( i18n( "Organization" ), KBibTeX::FieldLineEdit::itSingleLine, m_isReadOnly, this, "m_fieldLineEditOrganization" );
        label->setBuddy( m_fieldLineEditOrganization );
        gridLayout->addWidget( m_fieldLineEditOrganization, 0, 1 );

        label = new QLabel( QString( "%1:" ).arg( i18n( "Publisher" ) ), this );
        gridLayout->addWidget( label, 0, 3 );
        m_fieldLineEditPublisher = new KBibTeX::FieldLineEdit( i18n( "Publisher" ), KBibTeX::FieldLineEdit::itSingleLine, m_isReadOnly, this, "m_fieldLineEditPublisher" );
        label->setBuddy( m_fieldLineEditPublisher );
        gridLayout->addWidget( m_fieldLineEditPublisher, 0, 4 );

        label = new QLabel( QString( "%1:" ).arg( i18n( "School" ) ), this );
        gridLayout->addWidget( label, 1, 0 );
        m_fieldLineEditSchool = new KBibTeX::FieldLineEdit( i18n( "School" ), KBibTeX::FieldLineEdit::itSingleLine, m_isReadOnly, this, "m_fieldLineEditSchool" );
        label->setBuddy( m_fieldLineEditSchool );
        gridLayout->addWidget( m_fieldLineEditSchool, 1, 1 );

        label = new QLabel( QString( "%1:" ).arg( i18n( "Institution" ) ), this );
        gridLayout->addWidget( label, 1, 3 );
        m_fieldLineEditInstitution = new KBibTeX::FieldLineEdit( i18n( "Institution" ), KBibTeX::FieldLineEdit::itSingleLine, m_isReadOnly, this, "m_fieldLineEditInstitution" );
        label->setBuddy( m_fieldLineEditInstitution );
        gridLayout->addWidget( m_fieldLineEditInstitution, 1, 4 );

        label = new QLabel( QString( "%1:" ).arg( i18n( "Location" ) ), this );
        gridLayout->addWidget( label, 2, 0 );
        m_fieldLineEditLocation = new KBibTeX::FieldLineEdit( i18n( "Location" ), KBibTeX::FieldLineEdit::itSingleLine, m_isReadOnly, this, "m_fieldLineEditLocation" );
        label->setBuddy( m_fieldLineEditLocation );
        gridLayout->addWidget( m_fieldLineEditLocation, 2, 1 );

        label = new QLabel( QString( "%1:" ).arg( i18n( "Address" ) ), this );
        gridLayout->addWidget( label, 2, 3 );
        m_fieldLineEditAddress = new KBibTeX::FieldLineEdit( i18n( "Address" ), KBibTeX::FieldLineEdit::itSingleLine, m_isReadOnly, this, "m_fieldLineEditAddress" );
        label->setBuddy( m_fieldLineEditAddress );
        gridLayout->addWidget( m_fieldLineEditAddress, 2, 4 );

        label = new QLabel( QString( "%1:" ).arg( i18n( "Journal" ) ), this );
        gridLayout->addWidget( label, 3, 0 );
        m_fieldLineEditJournal = new KBibTeX::FieldLineEdit( i18n( "Journal" ), KBibTeX::FieldLineEdit::itSingleLine, m_isReadOnly, this, "m_fieldLineEditJournal" );
        label->setBuddy( m_fieldLineEditJournal );
        gridLayout->addWidget( m_fieldLineEditJournal, 3, 1 );

        label = new QLabel( QString( "%1:" ).arg( i18n( "Pages" ) ), this );
        gridLayout->addWidget( label, 3, 3 );
        m_fieldLineEditPages = new KBibTeX::FieldLineEdit( i18n( "Pages" ), KBibTeX::FieldLineEdit::itSingleLine, m_isReadOnly, this, "m_fieldLineEditPages" );
        label->setBuddy( m_fieldLineEditPages );
        gridLayout->addWidget( m_fieldLineEditPages, 3, 4 );

        label = new QLabel( QString( "%1:" ).arg( i18n( "Edition" ) ), this );
        gridLayout->addWidget( label, 4, 0 );
        m_fieldLineEditEdition = new KBibTeX::FieldLineEdit( i18n( "Edition" ), KBibTeX::FieldLineEdit::itSingleLine, m_isReadOnly, this, "m_fieldLineEditEdition" );
        label->setBuddy( m_fieldLineEditEdition );
        gridLayout->addWidget( m_fieldLineEditEdition, 4, 1 );

        label = new QLabel( QString( "%1:" ).arg( i18n( "Chapter" ) ), this );
        gridLayout->addWidget( label, 4, 3 );
        m_fieldLineEditChapter = new KBibTeX::FieldLineEdit( i18n( "Chapter" ), KBibTeX::FieldLineEdit::itSingleLine, m_isReadOnly, this, "m_fieldLineEditChapter" );
        label->setBuddy( m_fieldLineEditChapter );
        gridLayout->addWidget( m_fieldLineEditChapter, 4, 4 );

        label = new QLabel( QString( "%1:" ).arg( i18n( "Volume" ) ), this );
        gridLayout->addWidget( label, 5, 0 );
        m_fieldLineEditVolume = new KBibTeX::FieldLineEdit( i18n( "Volume" ), KBibTeX::FieldLineEdit::itSingleLine, m_isReadOnly, this, "m_fieldLineEditVolume" );
        label->setBuddy( m_fieldLineEditVolume );
        gridLayout->addWidget( m_fieldLineEditVolume, 5, 1 );

        label = new QLabel( QString( "%1:" ).arg( i18n( "Number" ) ), this );
        gridLayout->addWidget( label, 5, 3 );
        m_fieldLineEditNumber = new KBibTeX::FieldLineEdit( i18n( "Number" ), KBibTeX::FieldLineEdit::itSingleLine, m_isReadOnly, this, "m_fieldLineEditNumber" );
        label->setBuddy( m_fieldLineEditNumber );
        gridLayout->addWidget( m_fieldLineEditNumber, 5, 4 );

        label = new QLabel( QString( "%1:" ).arg( i18n( "Month" ) ), this );
        gridLayout->addWidget( label, 6, 0 );
        QHBoxLayout *innerLayout = new QHBoxLayout();
        gridLayout->addLayout( innerLayout, 6, 1 );
        m_pushButtonMonths = new QPushButton( this, "m_pushButtonMonths" );
        m_pushButtonMonths->setIconSet( QIconSet( SmallIcon( "vcalendar" ) ) );
        m_pushButtonMonths->setSizePolicy( QSizePolicy::Fixed, QSizePolicy::Minimum );
        innerLayout->addWidget( m_pushButtonMonths );
        m_fieldLineEditMonth = new KBibTeX::FieldLineEdit( i18n( "Month" ), KBibTeX::FieldLineEdit::itSingleLine, m_isReadOnly, this, "m_fieldLineEditMonth" );
        label->setBuddy( m_fieldLineEditMonth );
        innerLayout->addWidget( m_fieldLineEditMonth );

        label = new QLabel( QString( "%1:" ).arg( i18n( "Year" ) ), this );
        gridLayout->addWidget( label, 6, 3 );
        m_fieldLineEditYear = new KBibTeX::FieldLineEdit( i18n( "Year" ), KBibTeX::FieldLineEdit::itSingleLine, m_isReadOnly, this, "m_fieldLineEditYear" );
        label->setBuddy( m_fieldLineEditYear );
        gridLayout->addWidget( m_fieldLineEditYear, 6, 4 );

        label = new QLabel( QString( "%1:" ).arg( i18n( "ISBN" ) ), this );
        gridLayout->addWidget( label, 7, 0 );
        m_fieldLineEditISBN = new KBibTeX::FieldLineEdit( i18n( "ISBN" ), KBibTeX::FieldLineEdit::itSingleLine, m_isReadOnly, this, "m_fieldLineEditISBN" );
        label->setBuddy( m_fieldLineEditISBN );
        gridLayout->addWidget( m_fieldLineEditISBN, 7, 1 );

        label = new QLabel( QString( "%1:" ).arg( i18n( "ISSN" ) ), this );
        gridLayout->addWidget( label, 7, 3 );
        m_fieldLineEditISSN = new KBibTeX::FieldLineEdit( i18n( "ISSN" ), KBibTeX::FieldLineEdit::itSingleLine, m_isReadOnly, this, "m_fieldLineEditISSN" );
        label->setBuddy( m_fieldLineEditISSN );
        gridLayout->addWidget( m_fieldLineEditISSN, 7, 4 );

        label = new QLabel( QString( "%1:" ).arg( i18n( "Cross Reference" ) ), this );
        gridLayout->addWidget( label, 8, 0 );
        m_fieldLineEditCrossRef = new KBibTeX::FieldLineEdit( i18n( "Cross Reference" ), KBibTeX::FieldLineEdit::itSingleLine, m_isReadOnly, this, "m_fieldLineEditCrossRef" );
        label->setBuddy( m_fieldLineEditCrossRef );
        gridLayout->addWidget( m_fieldLineEditCrossRef, 8, 1 );


        label = new QLabel( QString( "%1:" ).arg( i18n( "How Published" ) ), this );
        gridLayout->addWidget( label, 8, 3 );
        m_fieldLineEditHowPublished = new KBibTeX::FieldLineEdit( i18n( "How Published" ), KBibTeX::FieldLineEdit::itSingleLine, m_isReadOnly, this, "m_fieldLineEditHowPublished" );
        label->setBuddy( m_fieldLineEditHowPublished );
        gridLayout->addWidget( m_fieldLineEditHowPublished, 8, 4 );

        QPopupMenu *menuMonths = new QPopupMenu( this, "menuMonths" );
        for ( int i = 0; i < 12;i++ )
            menuMonths->insertItem( BibTeX::Months[ i ], i );
        m_pushButtonMonths->setPopup( menuMonths );
        connect( menuMonths, SIGNAL( activated( int ) ), this, SLOT( slotSetMonth( int ) ) );
    }

}
#include "entrywidgetpublication.moc"
