/***************************************************************************
*   Copyright (C) 2004-2005 by Thomas Fischer                             *
*   fischer@unix-ag.uni-kl.de                                             *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#include <qlabel.h>
#include <qlineedit.h>
#include <qlayout.h>
#include <qbuffer.h>
#include <kdialogbase.h>
#include <klocale.h>

#include <bibtexstring.h>
#include <bibtexelement.h>
#include <bibtexfileexporterbibtex.h>
#include <bibtexfileimporterbibtex.h>
#include <fieldlineedit.h>

#include "stringwidget.h"

namespace KBibTeX
{
    QDialog::DialogCode StringWidget::execute( BibTeX::BibTeXString *string, bool isReadOnly, QWidget *parent, const char *name )
    {
        KDialogBase * dlg = new KDialogBase( parent, name, TRUE, i18n( "Edit BibTeX String" ), KDialogBase::Ok | KDialogBase::Cancel );
        StringWidget* stringWidget = new StringWidget( string, isReadOnly, dlg, "StringWidget" );

        dlg->setMainWidget( stringWidget );
        connect( dlg, SIGNAL( okClicked() ), stringWidget, SLOT( apply() ) );

        QDialog::DialogCode result = ( QDialog::DialogCode ) dlg->exec();

        delete( stringWidget );
        delete( dlg );

        return result;
    }

    StringWidget::StringWidget( BibTeX::BibTeXString *bibtexstring, bool isReadOnly, QWidget *parent, const char *name )
            : QTabWidget( parent, name ), m_isReadOnly( isReadOnly ), m_bibtexstring( bibtexstring )
    {
        setupGUI();
        reset();
    }

    StringWidget::~StringWidget()
    {
        // nothing
    }

    void StringWidget::apply()
    {
        m_bibtexstring->setKey( m_lineEditStringId->text() );
        BibTeX::BibTeXValue *value = m_fieldLineEditStringValue->value();
        qDebug( "value=%s", value->debugString().latin1() );
        m_bibtexstring->setValue( value );
    }

    void StringWidget::reset()
    {
        m_lineEditStringId->setText( m_bibtexstring->key() );
        m_fieldLineEditStringValue->setValue( m_bibtexstring->value() );
    }

    void StringWidget::setupGUI()
    {
        setMinimumWidth( 384 );

        QWidget *normalEdit = new QWidget( this, "normalEdit" );
        insertTab( normalEdit, i18n( "Normal" ) );
        QVBoxLayout * layout = new QVBoxLayout( normalEdit, KDialog::marginHint(), KDialog::spacingHint() );

        QLabel *label = new QLabel( i18n( "String &id:" ), normalEdit );
        layout->addWidget( label );
        m_lineEditStringId = new QLineEdit( normalEdit, "m_lineEditStringId" );
        m_lineEditStringId->setReadOnly( m_isReadOnly );
        layout->addWidget( m_lineEditStringId );
        label->setBuddy( m_lineEditStringId );

        label = new QLabel( i18n( "String &value:" ), normalEdit );
        layout->addWidget( label );
        m_fieldLineEditStringValue = new KBibTeX::FieldLineEdit( i18n( "String" ), KBibTeX::FieldLineEdit::itMultiLine, m_isReadOnly, normalEdit, "m_fieldLineEditStringValue" );
        layout->addWidget( m_fieldLineEditStringValue );
        label->setBuddy( m_fieldLineEditStringValue );
    }
}

#include "stringwidget.moc"
